@@ -2,7 +2,7 @@ package Graph::Geometric;
 
 use strict;
 use warnings;
-use feature qw(current_sub);
+use feature qw(current_sub); # Perl v5.16 or later
 
 =head1 NAME
 
@@ -11,7 +11,7 @@ Graph::Geometric - create and work with geometric graphs
 =cut
 
 # ABSTRACT: Create and work with geometric graphs
-our $VERSION = '0.1.0'; # VERSION
+our $VERSION = '0.2.0'; # VERSION
 
 use Exporter;
 use Graph 0.9713;
@@ -28,10 +28,12 @@ our @polygon_names = (
 );
 my @subs = qw(
     antiprism
+    bifrustum
     bipyramid
     cucurbituril
     cupola
     dodecahedron
+    gyrobicupola
     icosahedron
     icosidodecahedron
     octahedron
@@ -46,20 +48,29 @@ my @subs = qw(
     truncated
 );
 push @subs, map { $_ . 'gonal' } @polygon_names[1..$#polygon_names];
+push @subs, 'triangular', 'square';
 
 our @EXPORT = @subs;
 
-use List::Util qw( all sum );
+use List::Util qw( all any max sum uniq );
 use Set::Scalar;
 
 sub AUTOLOAD {
     our $AUTOLOAD;
     my $call = $AUTOLOAD;
     $call =~ s/.*:://;
-    return if $call eq 'DESTROY' || $call !~ /gonal$/;
-    my $N = _polygon_name_to_number( $call );
-    return unless defined $N;
-    $_[0]->($N);
+    return if $call eq 'DESTROY';
+    if( $call =~ /gonal$/ ) {
+        my $N = _polygon_name_to_number( $call );
+        return unless defined $N;
+        $_[0]->($N);
+    } elsif( $call eq 'triangular' ) {
+        $_[0]->(3);
+    } elsif( $call eq 'square' ) {
+        $_[0]->(4);
+    } else {
+        return;
+    }
 }
 
 =head1 SYNOPSIS
@@ -74,7 +85,7 @@ sub AUTOLOAD {
 
 =head1 DESCRIPTION
 
-C<Graph::Geometric> is an extension of C<Graph> to support working with geometric (topologic) graphs.
+C<Graph::Geometric> is an extension of L<Graph> to support working with geometric (topologic) graphs.
 In addition to vertices and edges, C<Graph::Geometric> has a concept of faces to ease handling of geometric graphs.
 C<Graph::Geometric> does not provide coordinates for vertices as it is supposed to be used for topology analysis of geometric shapes.
 
@@ -122,10 +133,62 @@ sub antiprism
         push @faces, Set::Scalar->new( map { $vertices[($i*2+$_) % ($N*2)] } 1..3 );
     }
 
+    $self->set_graph_attribute( 'constructor', 'antiprism' );
     $self->set_graph_attribute( 'faces', \@faces );
     return bless $self;
 }
 
+=head2 C<bifrustum( $N )>
+
+Given N, creates an N-gonal bifrustum.
+If N is not given, returns a code reference to itself.
+Since C<Graph::Geometric> does not provide coordinates, bifrustum is simply a prism with side faces cut in half.
+
+=cut
+
+sub bifrustum
+{
+    my( $N ) = @_;
+    return __SUB__ unless defined $N;
+
+    my $prism = prism( $N );
+
+    # Find top and bottom faces by the number of vertices in them
+    # FIXME: The following is unstable in cubes
+    my( $F1, $F2 ) = grep { scalar( @$_ ) == $N } $prism->faces;
+
+    # Select size edges, Graph::subgraph() does not work somewhy
+    my @side_edges;
+    for ($prism->edges) {
+        my $in_F1 = (Set::Scalar->new( @$F1 ) * Set::Scalar->new( @$_ ))->size;
+        my $in_F2 = (Set::Scalar->new( @$F2 ) * Set::Scalar->new( @$_ ))->size;
+        push @side_edges, $_ if $in_F1 && $in_F2;
+    }
+
+    # Carve side edges
+    for( @side_edges ) {
+        $prism->carve_edge( @$_ );
+    }
+
+    # Align vertices in both faces
+    my @F1 = $prism->_cycle_in_order( @$F1 );
+    my @F2;
+    for my $vertex (@F1) {
+        my( $edge ) = grep { $_->[0] eq $vertex || $_->[1] eq $vertex } @side_edges;
+        push @F2, grep { $_ ne $vertex } @$edge;
+    }
+
+    # Carve all side faces
+    for (0..($N-1)) {
+        $prism->carve_face( join( '', sort ( $F1[$_], $F2[$_] ) ),
+                            join( '', sort ( $F1[($_+1) % $N], $F2[($_+1) % $N] ) ) );
+    }
+
+    $prism->set_graph_attribute( 'constructor', 'bifrustum' );
+
+    return $prism;
+}
+
 =head2 C<bipyramid( $N )>
 
 Given N, creates an N-gonal bipyramid.
@@ -141,6 +204,7 @@ sub bipyramid
     my $pyramid = pyramid( $N );
     my( $base ) = $pyramid->faces;
     $pyramid->stellate( $base );
+    $pyramid->set_graph_attribute( 'constructor', 'bipyramid' );
     return $pyramid;
 }
 
@@ -192,7 +256,9 @@ sub cucurbituril
                Set::Scalar->new( @CAP2 ),
                @faces );
 
+    $self->set_graph_attribute( 'constructor', 'cucurbituril' );
     $self->set_graph_attribute( 'faces', \@faces );
+
     return bless $self;
 }
 
@@ -210,10 +276,11 @@ sub cupola
 
     my $prism = prism( $N*2 );
     my( $face ) = grep { scalar( @$_ ) == $N*2 } $prism->faces;
-    my @face = _face_in_order( $prism, @$face );
+    my @face = $prism->_cycle_in_order( @$face );
     while( @face ) {
         $prism->delete_edge( shift @face, shift @face );
     }
+    $prism->set_graph_attribute( 'constructor', 'cupola' );
     return $prism;
 }
 
@@ -227,9 +294,57 @@ sub dodecahedron()
 {
     my $pt = trapezohedron( 5 );
     $pt->truncate( 'A', 'B' );
+    $pt->set_graph_attribute( 'constructor', 'dodecahedron' );
     return $pt;
 }
 
+=head2 C<gyrobicupola( $N )>
+
+Given N, creates an N-gonal gyrobicupola.
+If N is not given, returns a code reference to itself.
+Implementation detail: gyrobicupola is constructed by creating a bifrustum and removing every second edge on top and bottom faces.
+
+=cut
+
+sub gyrobicupola
+{
+    my( $N ) = @_;
+    return __SUB__ unless defined $N;
+
+    my $bifrustum = bifrustum( $N*2 );
+
+    # All side vertices, and only them, have degree of 4
+    my @side_vertices =
+        $bifrustum->_cycle_in_order( grep { $bifrustum->degree( $_ ) == 4 }
+                                          $bifrustum->vertices );
+
+    # Top and bottom faces consist only of vertices of degree 3
+    my( $F1, $F2 ) = grep { all { $bifrustum->degree( $_ ) == 3 } @$_ }
+                          $bifrustum->faces;
+
+    # Collect face vertices in the same order
+    my( @F1, @F2 );
+    for my $vertex (@side_vertices) {
+        my( $v1, $v2 ) = grep { $bifrustum->degree( $_ ) == 3 }
+                              $bifrustum->neighbours( $vertex );
+        ( $v1, $v2 ) = ( $v2, $v1 ) if grep { $_ eq $v2 } @$F1;
+        push @F1, $v1;
+        push @F2, $v2;
+    }
+
+    push @F2, shift @F2;
+    while( @F1 ) {
+        $bifrustum->delete_edge( shift @F1, shift @F1 );
+    }
+    while( @F2 ) {
+        $bifrustum->delete_edge( shift @F2, shift @F2 );
+    }
+
+    $bifrustum->set_graph_attribute( 'constructor', 'gyrobicupola' );
+
+    return $bifrustum;
+}
+
 =head2 C<icosahedron()>
 
 Creates a regular icosahedron.
@@ -267,7 +382,7 @@ sub octahedron()
 
 Given N, creates an N-gonal orthobicupola.
 If N is not given, returns a code reference to itself.
-Implementation detail: orthobicupola is constructed by creating a prism, carving its side edges in half, joining the midpoints and removing every second edge on top and bottom faces.
+Implementation detail: orthobicupola is constructed by creating a bifrustum and removing every second edge on top and bottom faces.
 
 =cut
 
@@ -276,47 +391,36 @@ sub orthobicupola
     my( $N ) = @_;
     return __SUB__ unless defined $N;
 
-    my $prism = prism( $N*2 );
+    my $bifrustum = bifrustum( $N*2 );
 
-    # Find top and bottom faces by the number of vertices in them
-    # FIXME: The following is unstable in cubes
-    my( $F1, $F2 ) = grep { scalar( @$_ ) == $N*2 } $prism->faces;
+    # All side vertices, and only them, have degree of 4
+    my @side_vertices =
+        $bifrustum->_cycle_in_order( grep { $bifrustum->degree( $_ ) == 4 }
+                                          $bifrustum->vertices );
 
-    # Select size edges, Graph::subgraph() does not work somewhy
-    my @side_edges;
-    for ($prism->edges) {
-        my $in_F1 = (Set::Scalar->new( @$F1 ) * Set::Scalar->new( @$_ ))->size;
-        my $in_F2 = (Set::Scalar->new( @$F2 ) * Set::Scalar->new( @$_ ))->size;
-        push @side_edges, $_ if $in_F1 && $in_F2;
-    }
+    # Top and bottom faces consist only of vertices of degree 3
+    my( $F1, $F2 ) = grep { all { $bifrustum->degree( $_ ) == 3 } @$_ }
+                          $bifrustum->faces;
 
-    # Carve side edges
-    for( @side_edges ) {
-        $prism->carve_edge( @$_ );
-    }
-
-    # Align vertices in both faces
-    my @F1 = _face_in_order( $prism, @$F1 );
-    my @F2;
-    for my $vertex (@F1) {
-        my( $edge ) = grep { $_->[0] eq $vertex || $_->[1] eq $vertex } @side_edges;
-        push @F2, grep { $_ ne $vertex } @$edge;
-    }
-
-    # Carve all side faces
-    for (0..($N*2-1)) {
-        $prism->carve_face( join( '', sort ( $F1[$_], $F2[$_] ) ),
-                            join( '', sort ( $F1[($_+1) % ($N*2)], $F2[($_+1) % ($N*2)] ) ) );
+    # Collect face vertices in the same order
+    my( @F1, @F2 );
+    for my $vertex (@side_vertices) {
+        my( $v1, $v2 ) = grep { $bifrustum->degree( $_ ) == 3 }
+                              $bifrustum->neighbours( $vertex );
+        ( $v1, $v2 ) = ( $v2, $v1 ) if grep { $_ eq $v2 } @$F1;
+        push @F1, $v1;
+        push @F2, $v2;
     }
 
     while( @F1 ) {
-        $prism->delete_edge( shift @F1, shift @F1 );
+        $bifrustum->delete_edge( shift @F1, shift @F1 );
     }
     while( @F2 ) {
-        $prism->delete_edge( shift @F2, shift @F2 );
+        $bifrustum->delete_edge( shift @F2, shift @F2 );
     }
 
-    return $prism;
+    $bifrustum->set_graph_attribute( 'constructor', 'orthobicupola' );
+    return $bifrustum;
 }
 
 =head2 C<prism( $N )>
@@ -348,7 +452,9 @@ sub prism
                                        $F2[($_+1) % $N] );
     }
 
+    $self->set_graph_attribute( 'constructor', 'prism' );
     $self->set_graph_attribute( 'faces', \@faces );
+
     return bless $self;
 }
 
@@ -380,7 +486,9 @@ sub pyramid
                                        $base[($_+1) % $N] );
     }
 
+    $self->set_graph_attribute( 'constructor', 'pyramid' );
     $self->set_graph_attribute( 'faces', \@faces );
+
     return bless $self;
 }
 
@@ -416,6 +524,7 @@ sub rotunda
         last;
     }
     $cupola->face_dualify( @$face );
+    $cupola->set_graph_attribute( 'constructor', 'rotunda' );
 
     return $cupola;
 }
@@ -443,7 +552,9 @@ sub tetrahedron()
         push @faces, Set::Scalar->new( grep { $_ ne $v1 } @vertices );
     }
 
+    $self->set_graph_attribute( 'constructor', 'pyramid' );
     $self->set_graph_attribute( 'faces', \@faces );
+
     return bless $self; # TODO: Bless with class?
 }
 
@@ -479,6 +590,7 @@ sub trapezohedron
                                        map { $equator[($i*2+$_) % ($N*2)] } 1..3 );
     }
 
+    $self->set_graph_attribute( 'constructor', 'trapezohedron' );
     $self->set_graph_attribute( 'faces', \@faces );
     return bless $self; # TODO: Bless with class?
 }
@@ -565,7 +677,9 @@ sub delete_edge
         push @faces, $_ if $_->size > 2; # Exclude collapsed faces
     }
 
+    $self->delete_graph_attribute( 'constructor' );
     $self->set_graph_attribute( 'faces', \@faces );
+
     return $self;
 }
 
@@ -573,7 +687,7 @@ sub delete_edge
 
 Deletes a given face from polyhedra.
 A face is defined by an unordered list of vertices.
-Given face is assumed to exist, a check will be implemented later.
+Does nothing if a given face does not exist.
 Modifies and returns the original object.
 
 =cut
@@ -581,6 +695,7 @@ Modifies and returns the original object.
 sub delete_face
 {
     my( $self, $face ) = @_;
+    return $self unless $self->has_face( $face );
 
     my $vertex = join '', sort @$face;
     $self->_ensure_vertices_do_not_exist( $vertex );
@@ -603,7 +718,9 @@ sub delete_face
         $_->insert( $vertex );
     }
 
+    $self->delete_graph_attribute( 'constructor' );
     $self->set_graph_attribute( 'faces', \@faces );
+
     return $self;
 }
 
@@ -629,11 +746,26 @@ sub delete_vertex
     my $new_face = sum( @containing_faces );
     $new_face->delete( $vertex );
 
+    $self->delete_graph_attribute( 'constructor' );
     $self->set_graph_attribute( 'faces', [ @other_faces, $new_face ] );
 
     return $self;
 }
 
+=head2 C<has_face( $face )>
+
+Returns true if a given face exists, false otherwise.
+
+=cut
+
+sub has_face
+{
+    my( $self, $face ) = @_;
+    $face = join ',', sort @$face;
+    return any { join( ',', sort @$_ ) eq $face }
+               @{$self->get_graph_attribute( 'faces' )};
+}
+
 =head1 GEOMETRIC METHODS
 
 =cut
@@ -655,6 +787,8 @@ sub carve_edge
         $_->insert( $vertex );
     }
 
+    $self->delete_graph_attribute( 'constructor' );
+
     return $self;
 }
 =head2 C<carve_face( $u, $v )>
@@ -683,7 +817,7 @@ sub carve_face
         local $" = ' and ';
         die "there is no face having both @edge vertices\n";
     }
-    my @face = _face_in_order( $self, $face->members );
+    my @face = $self->_cycle_in_order( $face->members );
 
     $self->add_edge( @edge );
 
@@ -702,6 +836,150 @@ sub carve_face
     push @faces, Set::Scalar->new( @F1 ), Set::Scalar->new( @F2 );
     $self->set_graph_attribute( 'faces', \@faces );
 
+    $self->delete_graph_attribute( 'constructor' );
+
+    return $self;
+}
+
+=head2 C<elongate( $cycle )>
+
+Elongate the given polyhedron.
+If given a face, elongates the polyhedron by extruding it.
+If given a non-face cycle, elongates the polyhedron by cutting accross the given cycle and inserting a prism in between.
+If not given anything, attempts to elongate according to the type of given polyhedron.
+I.e., pyramids are elongated by extruding their faces and so on.
+Subroutine dies if it encounters a polyhedron of a type unknown to it.
+
+=cut
+
+sub elongate
+{
+    my( $self, $cycle ) = @_;
+
+    if( $cycle && $self->has_face( $cycle ) ) {
+        # Elongate along the given face
+        my @face = $self->_cycle_in_order( @$cycle );
+        my @vertices = map { $_ . 'e' } @face;
+        $self->_ensure_vertices_do_not_exist( @vertices );
+        $self->add_cycle( @vertices );
+
+        my @faces;
+        for my $face (@{$self->get_graph_attribute( 'faces' )}) {
+            next if join( '', sort $face->members ) eq join( '', sort @face );
+            push @faces, $face;
+        }
+
+        for my $i (0..$#face) {
+            $self->add_edge( $face[$i], $face[$i] . 'e' );
+            push @faces, Set::Scalar->new( $face[$i],
+                                           $face[$i] . 'e',
+                                           $face[($i+1) % @face],
+                                           $face[($i+1) % @face] . 'e' );
+        }
+        push @faces, Set::Scalar->new( @vertices );
+
+        $self->set_graph_attribute( 'faces', \@faces );
+    } elsif( $cycle ) {
+        # Elongate along the given cut
+        if( !$self->has_cycle( @$cycle ) ) {
+            die "cannot elongate nonexisting cycle\n";
+        }
+
+        my @cycle = $self->_cycle_in_order( @$cycle );
+        my $copy = $self->copy;
+        for (@cycle) { $copy->SUPER::delete_vertex( $_ ) } # delete_vertices() does not work
+        my( $C1, $C2 ) = $copy->connected_components; # FIXME: Establish deterministic order
+
+        my @vertices1 = map { $_ . '1' } @cycle;
+        my @vertices2 = map { $_ . '2' } @cycle; # FIXME: Check new vertices
+
+        $self->add_cycle( @vertices1 );
+        $self->add_cycle( @vertices2 );
+
+        my $cycle_set = Set::Scalar->new( @cycle );
+        for my $face (@{$self->get_graph_attribute( 'faces' )}) {
+            my @vertices = ($face * $cycle_set)->elements;
+            next unless @vertices;
+
+            # The remaining vertices in a face will belong either to $C1 or $C2
+            my( $vertex ) = ($face - $cycle_set)->elements;
+            if( any { $_ eq $vertex } @$C1 ) {
+                $face->invert( @vertices, map { $_ . '1' } @vertices );
+            } else {
+                $face->invert( @vertices, map { $_ . '2' } @vertices );
+            }
+        }
+
+        my @new_faces;
+        for (0..$#cycle) {
+            $self->add_edge( $vertices1[$_], $vertices2[$_] );
+            push @new_faces, Set::Scalar->new( $vertices1[$_],
+                                               $vertices2[$_],
+                                               $vertices1[($_+1) % @cycle],
+                                               $vertices2[($_+1) % @cycle] );
+        }
+
+        for my $cycle_vertex (@cycle) {
+            for my $neighbour ($self->neighbours( $cycle_vertex )) {
+                if(      any { $_ eq $neighbour } @$C1 ) {
+                    $self->add_edge( $neighbour, $cycle_vertex . '1' );
+                } elsif( any { $_ eq $neighbour } @$C2 ) {
+                    $self->add_edge( $neighbour, $cycle_vertex . '2' );
+                }
+            }
+        }
+
+        for (@cycle) { $self->SUPER::delete_vertex( $_ ) } # delete_vertices() does not work
+        $self->set_graph_attribute( 'faces', [ @{$self->get_graph_attribute( 'faces' )},
+                                               @new_faces ] );
+    } else {
+        # Elongate according to the type of geometric figure
+        if( !$self->has_graph_attribute( 'constructor' ) ) {
+            die "unknown geometric type for elongation\n";
+        }
+
+        my $constructor = $self->get_graph_attribute( 'constructor' );
+
+        if(      $constructor eq 'bipyramid' ) {
+            # FIXME: May be nondeterministic for square bipyramids
+            my @cycle = grep { $self->degree( $_ ) == 4 } $self->vertices;
+            $self->elongate( \@cycle );
+        } elsif( $constructor eq 'cupola' ||
+                 $constructor eq 'pyramid' ||
+                 $constructor eq 'rotunda' ) {
+            # Elongate (extrude) the largest face
+            my( $face ) = sort { scalar( @$b ) <=> scalar( @$a ) } $self->faces;
+            $self->elongate( $face );
+        } elsif( $constructor eq 'gyrobicupola' ||
+                 $constructor eq 'orthobicupola' ) {
+            # Bicupolae are elongated along cycle where every vertex belongs to two triangular faces
+            my %triangular_faces;
+            for my $face ($self->faces) {
+                next unless scalar( @$face ) == 3;
+                for (@$face) {
+                    $triangular_faces{$_} = 0 unless $triangular_faces{$_};
+                    $triangular_faces{$_}++;
+                }
+            }
+
+            my @cycle = grep { $triangular_faces{$_} == 2 } keys %triangular_faces;
+            $self->elongate( \@cycle );
+        } elsif( $constructor eq 'gyrobirotunda' ||
+                $constructor eq 'orthobirotunda' ) {
+            # Birotundas should have two faces that are neither triangles nor pentagons.
+            # FIXME: Will fail with trigonal and pentagonal birotundas.
+            my @bases = grep { scalar( @$_ ) != 3 && scalar( @$_ ) != 5 }
+                             $self->faces;
+            my @removed = uniq @bases, map { $self->neighbours( $_ ) } @bases;
+            my $copy = $self->copy;
+            for (@removed) { $copy->SUPER::delete_vertex( $_ ) }
+            $self->elongate( [ $copy->vertices ] );
+        } else {
+            die "do not know how to elongate $constructor\n";
+        }
+    }
+
+    $self->delete_graph_attribute( 'constructor' );
     return $self;
 }
 
@@ -716,7 +994,7 @@ sub face_dualify
         push @faces, $face;
     }
 
-    @face = _face_in_order( $self, @face );
+    @face = $self->_cycle_in_order( @face );
     my @new_vertices;
     for (0..$#face) {
         my @edge = sort ( $face[$_], $face[($_+1) % scalar @face] );
@@ -729,6 +1007,9 @@ sub face_dualify
     push @faces, Set::Scalar->new( @new_vertices );
 
     $self->set_graph_attribute( 'faces', \@faces );
+    $self->delete_graph_attribute( 'constructor' );
+
+    return $self;
 }
 
 =head2 C<rectify()>
@@ -762,10 +1043,12 @@ sub rectify
         $self->delete_vertex( $vertex );
     }
 
+    $self->delete_graph_attribute( 'constructor' );
+
     return $self;
 }
 
-=head2 C<stellate()>
+=head2 C<stellate( @faces )>
 
 Given a polyhedron and a list of faces, performs stellation of specified faces.
 If no faces are given, all faces are stellated.
@@ -798,10 +1081,13 @@ sub stellate
         }
     }
 
+    $self->delete_graph_attribute( 'constructor' );
     $self->set_graph_attribute( 'faces', \@faces_now );
+
+    return $self;
 }
 
-=head2 C<truncate()>
+=head2 C<truncate( @vertices )>
 
 Given a polyhedron and a list of vertices, performs truncation of specified vertices.
 If no vertices are given, all vertices are stellated.
@@ -844,6 +1130,8 @@ sub truncate
         $self->SUPER::delete_vertex( $vertex );
     }
 
+    $self->delete_graph_attribute( 'constructor' );
+
     return $self;
 }
 
@@ -910,6 +1198,18 @@ They allow for such syntactically sweet calls like:
 
     my $truncated_icosahedron = truncated icosahedron;
 
+=head2 C<elongated()>
+
+Copies the given polyhedron, elongates it and returns the elongated polyhedron.
+This subroutine does not accept any parameters, thus if nonstandard elongation is wanted, other methods have to be used.
+
+=cut
+
+sub elongated($)
+{
+    return $_[0]->deep_copy->elongate;
+}
+
 =head2 C<rectified()>
 
 Copies the given polyhedron, rectifies it and returns the rectified polyhedron.
@@ -943,15 +1243,80 @@ sub truncated($)
     return $_[0]->deep_copy->truncate;
 }
 
-sub _ensure_vertices_do_not_exist
+=head1 PROPERTIES
+
+=head2 C<is_isogonal()>
+
+Tests if a given polyhedron is isogonal (vertex-transitive) by checking whether all its vertices belong to the same symmetry orbit.
+Requires L<Graph::Nauty> to locate symmetry orbits of a graph.
+
+=cut
+
+sub is_isogonal
 {
-    my( $graph, @vertices ) = @_;
-    for (@vertices) {
-        die "vertex $_ already exists\n" if $graph->has_vertex( $_ );
-    }
+    my( $self ) = @_;
+    require Graph::Nauty;
+    return scalar( Graph::Nauty::orbits( $self, sub { return '' } ) ) == 1;
 }
 
-sub _face_in_order
+=head2 C<is_isotoxal()>
+
+Tests if a given polyhedron is isotoxal (edge-transitive) by checking whether all its edges belong to the same symmetry orbit.
+Requires L<Graph::Nauty> to locate symmetry orbits of a graph.
+Implementation detail: figure is rectified and tested for being isogonal.
+
+=cut
+
+sub is_isotoxal
+{
+    my( $self ) = @_;
+    return $self->rectified->is_isogonal;
+}
+
+=head2 C<is_isohedral()>
+
+Tests if a given polyhedron is isohedral (face-transitive) by checking whether all its faces belong to the same symmetry orbit.
+Requires L<Graph::Nauty> to locate symmetry orbits of a graph.
+Implementation detail: dual of a figure is tested for being isogonal.
+
+=cut
+
+sub is_isohedral
+{
+    my( $self ) = @_;
+    return $self->dual->is_isogonal;
+}
+
+=head2 C<is_regular()>
+
+Tests if given polyhedron is regular.
+A regular polyhedron is vertex-transitive, edge-transitive and face-transitive.
+Requires L<Graph::Nauty> to locate symmetry orbits of a graph.
+
+=cut
+
+sub is_regular
+{
+    my( $self ) = @_;
+    return $self->is_isogonal && $self->is_isotoxal && $self->is_isohedral;
+}
+
+=head2 C<is_quasiregular()>
+
+Tests if given quasiregular is regular.
+A quasiregular polyhedron is vertex-transitive and edge-transitive, has two kinds of faces.
+Requires L<Graph::Nauty> to locate symmetry orbits of a graph.
+
+=cut
+
+sub is_quasiregular
+{
+    my( $self ) = @_;
+    return $self->is_isogonal && $self->is_isotoxal &&
+           scalar( Graph::Nauty::orbits( $self->dual, sub { return '' } ) ) == 2;
+}
+
+sub _cycle_in_order
 {
     my( $graph, @face ) = @_;
     my $subgraph = $graph->subgraph( \@face );
@@ -966,6 +1331,25 @@ sub _face_in_order
     return @order;
 }
 
+sub _ensure_vertices_do_not_exist
+{
+    my( $graph, @vertices ) = @_;
+    for (@vertices) {
+        die "vertex $_ already exists\n" if $graph->has_vertex( $_ );
+    }
+}
+
+sub _largest_face
+{
+    my( $self ) = @_;
+    my $max;
+    for ($self->faces) {
+        next if defined $max && scalar( @$_ ) <= scalar( @$max );
+        $max = $_;
+    }
+    return @$max;
+}
+
 sub _names
 {
     my( $N ) = @_;
